@@ -1,33 +1,71 @@
 import logging
-import sys
-from typing import Callable, Coroutine, Dict, List, Optional
+from typing import Callable, Coroutine, Dict, List, Literal, Optional, TypedDict
 from warnings import warn
 
 from dbus_fast import Variant
 
-if sys.version_info[:2] < (3, 8):
-    from typing_extensions import Literal, TypedDict
-else:
-    from typing import Literal, TypedDict
-
 from ...exc import BleakError
-from ..device import BLEDevice
 from ..scanner import AdvertisementData, AdvertisementDataCallback, BaseBleakScanner
 from .advertisement_monitor import OrPatternLike
 from .defs import Device1
 from .manager import get_global_bluez_manager
+from .utils import bdaddr_from_device_path
 
 logger = logging.getLogger(__name__)
 
 
 class BlueZDiscoveryFilters(TypedDict, total=False):
+    """
+    Dictionary of arguments for the ``org.bluez.Adapter1.SetDiscoveryFilter``
+    D-Bus method.
+
+    https://github.com/bluez/bluez/blob/master/doc/org.bluez.Adapter.rst#void-setdiscoveryfilterdict-filter
+    """
+
     UUIDs: List[str]
+    """
+    Filter by service UUIDs, empty means match _any_ UUID.
+
+    Normally, the ``service_uuids`` argument of :class:`bleak.BleakScanner`
+    is used instead.
+    """
     RSSI: int
+    """
+    RSSI threshold value.
+    """
     Pathloss: int
+    """
+    Pathloss threshold value.
+    """
     Transport: str
+    """
+    Transport parameter determines the type of scan.
+
+    This should not be used since it is required to be set to ``"le"``.
+    """
     DuplicateData: bool
+    """
+    Disables duplicate detection of advertisement data.
+
+    This does not affect the ``Filter Duplicates`` parameter of the ``LE Set Scan Enable``
+    HCI command to the Bluetooth adapter!
+
+    Although the default value for BlueZ is ``True``, Bleak sets this to ``False`` by default.
+    """
     Discoverable: bool
+    """
+    Make adapter discoverable while discovering,
+    if the adapter is already discoverable setting
+    this filter won't do anything.
+    """
     Pattern: str
+    """
+    Discover devices where the pattern matches
+    either the prefix of the address or
+    device name which is convenient way to limited
+    the number of device objects created during a
+    discovery.
+    """
 
 
 class BlueZScannerArgs(TypedDict, total=False):
@@ -55,7 +93,7 @@ class BleakScannerBlueZDBus(BaseBleakScanner):
 
     For possible values for `filters`, see the parameters to the
     ``SetDiscoveryFilter`` method in the `BlueZ docs
-    <https://git.kernel.org/pub/scm/bluetooth/bluez.git/tree/doc/adapter-api.txt?h=5.48&id=0d1e3b9c5754022c779da129025d493a198d49cf>`_
+    <https://github.com/bluez/bluez/blob/master/doc/org.bluez.Adapter.rst#void-setdiscoveryfilterdict-filter>`_
 
     Args:
         detection_callback:
@@ -89,9 +127,6 @@ class BleakScannerBlueZDBus(BaseBleakScanner):
         # kwarg "device" is for backwards compatibility
         self._adapter: Optional[str] = kwargs.get("adapter", kwargs.get("device"))
 
-        # map of d-bus object path to d-bus object properties
-        self._devices: Dict[str, Device1] = {}
-
         # callback from manager for stopping scanning if it has been started
         self._stop: Optional[Callable[[], Coroutine]] = None
 
@@ -129,7 +164,7 @@ class BleakScannerBlueZDBus(BaseBleakScanner):
         if self._scanning_mode == "passive" and not self._or_patterns:
             raise BleakError("passive scanning mode requires bluez or_patterns")
 
-    async def start(self):
+    async def start(self) -> None:
         manager = await get_global_bluez_manager()
 
         if self._adapter:
@@ -137,7 +172,7 @@ class BleakScannerBlueZDBus(BaseBleakScanner):
         else:
             adapter_path = manager.get_default_adapter()
 
-        self._devices.clear()
+        self.seen_devices = {}
 
         if self._scanning_mode == "passive":
             self._stop = await manager.passive_scan(
@@ -154,19 +189,19 @@ class BleakScannerBlueZDBus(BaseBleakScanner):
                 self._handle_device_removed,
             )
 
-    async def stop(self):
+    async def stop(self) -> None:
         if self._stop:
             # avoid reentrancy
             stop, self._stop = self._stop, None
 
             await stop()
 
-    def set_scanning_filter(self, **kwargs):
+    def set_scanning_filter(self, **kwargs) -> None:
         """Sets OS level scanning filters for the BleakScanner.
 
         For possible values for `filters`, see the parameters to the
         ``SetDiscoveryFilter`` method in the `BlueZ docs
-        <https://git.kernel.org/pub/scm/bluetooth/bluez.git/tree/doc/adapter-api.txt?h=5.48&id=0d1e3b9c5754022c779da129025d493a198d49cf>`_
+        <https://github.com/bluez/bluez/blob/master/doc/org.bluez.Adapter.rst#void-setdiscoveryfilterdict-filter>`_
 
         See variant types here: <https://python-dbus-next.readthedocs.io/en/latest/type-system/>
 
@@ -192,25 +227,6 @@ class BleakScannerBlueZDBus(BaseBleakScanner):
             else:
                 logger.warning("Filter '%s' is not currently supported." % k)
 
-    @property
-    def discovered_devices(self) -> List[BLEDevice]:
-        discovered_devices = []
-
-        for path, props in self._devices.items():
-            uuids = props.get("UUIDs", [])
-            manufacturer_data = props.get("ManufacturerData", {})
-            discovered_devices.append(
-                BLEDevice(
-                    props["Address"],
-                    props["Alias"],
-                    {"path": path, "props": props},
-                    props.get("RSSI", 0),
-                    uuids=uuids,
-                    manufacturer_data=manufacturer_data,
-                )
-            )
-        return discovered_devices
-
     # Helper methods
 
     def _handle_advertising_data(self, path: str, props: Device1) -> None:
@@ -222,11 +238,6 @@ class BleakScannerBlueZDBus(BaseBleakScanner):
             props: The D-Bus object properties of the device.
         """
 
-        self._devices[path] = props
-
-        if self._callback is None:
-            return
-
         # Get all the information wanted to pack in the advertisement data
         _local_name = props.get("Name")
         _manufacturer_data = {
@@ -244,29 +255,29 @@ class BleakScannerBlueZDBus(BaseBleakScanner):
             manufacturer_data=_manufacturer_data,
             service_data=_service_data,
             service_uuids=_service_uuids,
-            platform_data=props,
             tx_power=tx_power,
+            rssi=props.get("RSSI", -127),
+            platform_data=(path, props),
         )
 
-        device = BLEDevice(
+        device = self.create_or_update_device(
             props["Address"],
             props["Alias"],
             {"path": path, "props": props},
-            props.get("RSSI", 0),
-            uuids=_service_uuids,
-            manufacturer_data=_manufacturer_data,
+            advertisement_data,
         )
 
-        self._callback(device, advertisement_data)
+        self.call_detection_callbacks(device, advertisement_data)
 
     def _handle_device_removed(self, device_path: str) -> None:
         """
         Handles a device being removed from BlueZ.
         """
         try:
-            del self._devices[device_path]
+            bdaddr = bdaddr_from_device_path(device_path)
+            del self.seen_devices[bdaddr]
         except KeyError:
-            # The device will not have been added to self._devices if no
+            # The device will not have been added to self.seen_devices if no
             # advertising data was received, so this is expected to happen
             # occasionally.
             pass
