@@ -130,6 +130,11 @@ class BleakScannerCoreBluetooth(BaseBleakScanner):
                 address_bytes: bytes = (
                     self._manager.central_manager.retrieveAddressForPeripheral_(p)
                 )
+                if address_bytes is None:
+                    logger.debug(
+                        "Could not get Bluetooth address for %s. Ignoring this device.",
+                        p.identifier().UUIDString(),
+                    )
                 address = address_bytes.hex(":").upper()
             else:
                 address = p.identifier().UUIDString()
